@@ -6,7 +6,7 @@
 //
 
 /*
-   Copyright or © or Copr. CNRS, (November 17, 2004)
+   Copyright or © or Copr. Bio++ Development Team, (November 17, 2004)
 
    This software is a computer program whose purpose is to provide classes
    for sequences analysis.
@@ -79,12 +79,24 @@ Alphabet* SequenceApplicationTools::getAlphabet(
     alphabet = args["letter"];
     flag = 1;
   }
+  else if (alphabet == "RNY")
+    {
+      if (args.find("letter") == args.end())
+        throw Exception("Missing letter alphabet for RNY alphabet");
+      alphabet = args["letter"];
+      flag = 2;
+    }
 
-  if (alphabet == "DNA")
-    chars = new DNA();
-  else if (alphabet == "RNA")
-    chars = new RNA();
-  else if (alphabet == "Protein")
+
+  if (alphabet == "Binary")
+    chars = new BinaryAlphabet();
+  else if (alphabet == "DNA") {
+    bool mark = ApplicationTools::getBooleanParameter("bangAsGap", args, false, "", true, false);
+    chars = new DNA(mark);
+  } else if (alphabet == "RNA") {
+    bool mark = ApplicationTools::getBooleanParameter("bangAsGap", args, false, "", true, false);
+    chars = new RNA(mark);
+  } else if (alphabet == "Protein")
     chars = new ProteicAlphabet();
   else if (allowGeneric && alphabet == "Generic")
     chars = new DefaultAlphabet();
@@ -93,14 +105,19 @@ Alphabet* SequenceApplicationTools::getAlphabet(
     if (args.find("letter") == args.end())
       throw Exception("Missing alphabet in Codon : " + alphabet);
 
-    string alphn = ApplicationTools::getStringParameter("letter", args, "RNA");
+    string alphnDesc = ApplicationTools::getStringParameter("letter", args, "RNA");
+    string alphn;
+    map<string, string> alphnArgs;
+    KeyvalTools::parseProcedure(alphnDesc, alphn, alphnArgs);
 
     NucleicAlphabet* pnalph;
-    if (alphn == "RNA")
-      pnalph = new RNA();
-    else if (alphn == "DNA")
-      pnalph = new DNA();
-    else
+    if (alphn == "RNA") {
+      bool mark = ApplicationTools::getBooleanParameter("bangAsGap", alphnArgs, false, "", true, false);
+      pnalph = new RNA(mark);
+    } else if (alphn == "DNA") {
+      bool mark = ApplicationTools::getBooleanParameter("bangAsGap", alphnArgs, false, "", true, false);
+      pnalph = new DNA(mark);
+    } else
       throw Exception("Alphabet not known in Codon : " + alphn);
 
     string type = ApplicationTools::getStringParameter("type", args, "Standard");
@@ -130,6 +147,17 @@ Alphabet* SequenceApplicationTools::getAlphabet(
     }
     alphabet = "Word(" + al + ")";
   }
+  else if (flag == 2)
+    {
+      if (AlphabetTools::isNucleicAlphabet(chars))
+        {
+          chars = new RNY(*(dynamic_cast<NucleicAlphabet*>(chars)));
+          alphabet = "RNY(" + alphabet + ")";
+        }
+      else
+        throw Exception("RNY needs a Nucleic Alphabet, instead of " + alphabet);
+    }
+
 
   if (verbose)
     ApplicationTools::displayResult("Alphabet type ", alphabet);
@@ -245,7 +273,9 @@ SequenceContainer* SequenceApplicationTools::getSequenceContainer(
   }
   else if (format == "Fasta")
   {
-    iSeq.reset(new Fasta());
+    bool strictNames = ApplicationTools::getBooleanParameter("strict_names", args, false, "", true, false);
+    bool extended    = ApplicationTools::getBooleanParameter("extended", args, false, "", true, false);
+    iSeq.reset(new Fasta(100, true, extended, strictNames));
   }
   else if (format == "Clustal")
   {
@@ -343,7 +373,9 @@ VectorSiteContainer* SequenceApplicationTools::getSiteContainer(
   }
   else if (format == "Fasta")
   {
-    iSeq.reset(new Fasta());
+    bool strictNames = ApplicationTools::getBooleanParameter("strict_names", args, false, "", true, false);
+    bool extended    = ApplicationTools::getBooleanParameter("extended", args, false, "", true, false);
+    iSeq.reset(new Fasta(100, true, extended, strictNames));
   }
   else if (format == "Clustal")
   {
@@ -362,13 +394,33 @@ VectorSiteContainer* SequenceApplicationTools::getSiteContainer(
   {
     throw Exception("Unknown sequence format: " + format);
   }
+  const Alphabet* alpha2;
+  if (AlphabetTools::isRNYAlphabet(alpha))
+    alpha2 = &dynamic_cast<const RNY*>(alpha)->getLetterAlphabet();
+  else
+    alpha2 = alpha;
 
-  const SequenceContainer* seqCont = iSeq->read(sequenceFilePath, alpha);
-  VectorSiteContainer* sites = new VectorSiteContainer(*dynamic_cast<const OrderedSequenceContainer*>(seqCont));
+  const SequenceContainer* seqCont = iSeq->read(sequenceFilePath, alpha2);
+
+  VectorSiteContainer* sites2 = new VectorSiteContainer(*dynamic_cast<const OrderedSequenceContainer*>(seqCont));
   delete seqCont;
 
-  if (verbose)
-    ApplicationTools::displayResult("Sequence file " + suffix, sequenceFilePath);
+  VectorSiteContainer* sites;
+
+  if (AlphabetTools::isRNYAlphabet(alpha))
+    {
+      const SequenceTools ST;
+      sites = new VectorSiteContainer(alpha);
+      for (unsigned int i = 0; i < sites2->getNumberOfSequences(); i++)
+        {
+          sites->addSequence(*(ST.RNYslice(sites2->getSequence(i))));
+          }
+      delete sites2;
+      }
+  else
+    sites = sites2;
+
+  if (verbose) ApplicationTools::displayResult("Sequence file " + suffix, sequenceFilePath);
 
   // Look for site selection:
   if (format == "Mase")
