@@ -6,7 +6,7 @@
 //
 
 /*
-   Copyright or © or Copr. CNRS, (November 17, 2004)
+   Copyright or © or Copr. Bio++ Development Team, (November 17, 2004)
 
    This software is a computer program whose purpose is to provide classes
    for sequences analysis.
@@ -476,8 +476,10 @@ throw (AlphabetMismatchException)
   if (seq1.getAlphabet()->getAlphabetType() != s.getAlphabet()->getAlphabetType())
     throw AlphabetMismatchException("SiteContainerTools::alignNW", seq1.getAlphabet(), s.getAlphabet());
   // Check that sequences have no gap!
-  Sequence* s1 = SequenceTools::removeGaps(seq1);
-  Sequence* s2 = SequenceTools::removeGaps(seq2);
+  auto_ptr<Sequence> s1(seq1.clone());
+  SequenceTools::removeGaps(*s1);
+  auto_ptr<Sequence> s2(seq2.clone());
+  SequenceTools::removeGaps(*s2);
 
   // 1) Initialize matrix:
   RowMatrix<double> m(s1->size() + 1, s2->size() + 1);
@@ -557,8 +559,6 @@ throw (AlphabetMismatchException)
   AlignedSequenceContainer* asc = new AlignedSequenceContainer(s1->getAlphabet());
   asc->addSequence(*s1, false);
   asc->addSequence(*s2, false); // Do not check for sequence names.
-  delete s1; // Sequence are recopied in the container, we can delete these...
-  delete s2;
   return asc;
 }
 
@@ -577,8 +577,10 @@ throw (AlphabetMismatchException)
   if (seq1.getAlphabet()->getAlphabetType() != s.getAlphabet()->getAlphabetType())
     throw AlphabetMismatchException("SiteContainerTools::alignNW", seq1.getAlphabet(), s.getAlphabet());
   // Check that sequences have no gap!
-  Sequence* s1 = SequenceTools::removeGaps(seq1);
-  Sequence* s2 = SequenceTools::removeGaps(seq2);
+  auto_ptr<Sequence> s1(seq1.clone());
+  SequenceTools::removeGaps(*s1);
+  auto_ptr<Sequence> s2(seq2.clone());
+  SequenceTools::removeGaps(*s2);
 
   // 1) Initialize matrix:
   RowMatrix<double> m(s1->size() + 1, s2->size() + 1);
@@ -695,8 +697,6 @@ throw (AlphabetMismatchException)
   AlignedSequenceContainer* asc = new AlignedSequenceContainer(s1->getAlphabet());
   asc->addSequence(*s1, false);
   asc->addSequence(*s2, false); // Do not check for sequence names.
-  delete s1; // Sequence are recopied in the container, we can delete these...
-  delete s2;
   return asc;
 }
 
@@ -874,3 +874,118 @@ throw (AlphabetMismatchException, Exception)
 }
 
 /******************************************************************************/
+
+void SiteContainerTools::getSequencePositions(const SiteContainer& sites, Matrix<unsigned int>& positions)
+{
+  positions.resize(sites.getNumberOfSequences(), sites.getNumberOfSites());
+  int gap = sites.getAlphabet()->getGapCharacterCode();
+  for (unsigned int i = 0; i < sites.getNumberOfSequences(); ++i) {
+    const Sequence& seq = sites.getSequence(i);
+    unsigned int pos = 0;
+    for (unsigned int j = 0; j < sites.getNumberOfSites(); ++j) {
+      if (seq[j] != gap) {
+        ++pos;
+        positions(i, j) = pos;
+      } else {
+        positions(i, j) = 0;
+      }
+    }
+  }
+}
+
+/******************************************************************************/
+
+vector<int> SiteContainerTools::getColumnScores(const Matrix<unsigned int>& positions1, const Matrix<unsigned int>& positions2, int na)
+{
+  if (positions1.getNumberOfRows() != positions2.getNumberOfRows())
+    throw Exception("SiteContainerTools::getColumnScores. The two input alignments must have the same number of sequences!");
+  vector<int> scores(positions1.getNumberOfColumns());
+  for (unsigned int i = 0; i < positions1.getNumberOfColumns(); ++i) {
+    //Find an anchor point:
+    unsigned int whichSeq = 0;
+    unsigned int whichPos = 0;
+    for (unsigned int j = 0; j < positions1.getNumberOfRows(); ++j) {
+      if (positions1(j, i) > 0) {
+        whichSeq = j;
+        whichPos = positions1(j, i);
+        break;
+      }
+    }
+    if (whichPos == 0) {
+      //No anchor found, this alignment column is only made of gaps. We assign a score of 'na' and move to the next column.
+      scores[i] = na;
+      continue;
+    }
+    //We look for the anchor in the reference alignment:
+    unsigned int i2 = 0;
+    bool found = false;
+    for (unsigned int j = 0; !found && j < positions2.getNumberOfColumns(); ++j) {
+      if (positions2(whichSeq, j) == whichPos) {
+        i2 = j;
+        found = true;
+      }
+    }
+    if (!found) {
+      throw Exception("SiteContainerTools::getColumnScores(). Position " + TextTools::toString(whichPos) + " of sequence " + TextTools::toString(whichSeq) + " not found in reference alignment. Please make sure the two indexes are built from the same data!");
+    }
+    //Now we compare all pairs of sequences between the two positions:
+    bool test = true;
+    for (unsigned int j = 0; test && j < positions1.getNumberOfRows(); ++j) {
+      test = (positions1(j, i) == positions2(j, i2));
+    }
+    scores[i] = test ? 1 : 0;
+  }
+  return scores;
+}
+
+/******************************************************************************/
+
+vector<double> SiteContainerTools::getSumOfPairsScores(const Matrix<unsigned int>& positions1, const Matrix<unsigned int>& positions2, double na)
+{
+  if (positions1.getNumberOfRows() != positions2.getNumberOfRows())
+    throw Exception("SiteContainerTools::getColumnScores. The two input alignments must have the same number of sequences!");
+  vector<double> scores(positions1.getNumberOfColumns());
+  for (unsigned int i = 0; i < positions1.getNumberOfColumns(); ++i) {
+    //For all positions in alignment 1...
+    unsigned int countAlignable = 0;
+    unsigned int countAligned   = 0;
+    for (unsigned int j = 0; j < positions1.getNumberOfRows(); ++j) {
+      //Get the corresponding column in alignment 2:
+      unsigned int whichPos = positions1(j, i);
+      if (whichPos == 0) {
+        //No position for this sequence here.
+        continue;
+      }
+      //We look for the position in the second alignment:
+      unsigned int i2 = 0;
+      bool found = false;
+      for (unsigned int k = 0; !found && k < positions2.getNumberOfColumns(); ++k) {
+        if (positions2(j, k) == whichPos) {
+          i2 = k;
+          found = true;
+        }
+      }
+      if (!found) {
+        throw Exception("SiteContainerTools::getColumnScores(). Position " + TextTools::toString(whichPos) + " of sequence " + TextTools::toString(j) + " not found in reference alignment. Please make sure the two indexes are built from the same data!");
+      }
+
+      //Now we check all other positions and see if they are aligned with this one:
+      for (unsigned int k = j + 1; k < positions1.getNumberOfRows(); ++k) {
+        unsigned int whichPos2 = positions1(k, i);
+        if (whichPos2 == 0) {
+          //Empty position
+          continue;
+        }
+        countAlignable++;
+        //check position in alignment 2:
+        if (positions2(k, i2) == whichPos2)
+          countAligned++;
+      }
+    }
+    scores[i] = countAlignable == 0 ? na : static_cast<double>(countAligned) / static_cast<double>(countAlignable);
+  }
+  return scores;
+}
+
+/******************************************************************************/
+
