@@ -7,7 +7,7 @@
 //
 
 /*
-Copyright or © or Copr. CNRS, (November 17, 2004)
+Copyright or © or Copr. Bio++ Development Team, (November 17, 2004)
 
 This software is a computer program whose purpose is to provide classes
 for sequences analysis.
@@ -58,6 +58,7 @@ using namespace std;
 
 DNA SequenceTools::_DNA;
 RNA SequenceTools::_RNA;
+RNY SequenceTools::_RNY(_DNA);
 NucleicAcidsReplication SequenceTools::_DNARep(& _DNA, & _DNA);
 NucleicAcidsReplication SequenceTools::_RNARep(& _RNA, & _RNA);
 NucleicAcidsReplication SequenceTools::_transc(& _DNA, & _RNA);
@@ -195,13 +196,13 @@ Sequence& SequenceTools::invertComplement(Sequence& seq) {
 	} else {
 		throw AlphabetException("SequenceTools::complement: Sequence must be nucleic.", seq.getAlphabet());
 	}
-  for (unsigned int i = 0 ; i < seq.size() ; i++) {
-    seq.setElement(i, NAR->translate(seq.getValue(i)));
-  }
-  unsigned int seq_size = seq.size(); // store seq size for efficiency
-  unsigned int tmp_state = 0; // to store one state when swapping positions
-  unsigned int j = seq_size; // symetric position iterator from sequence end
-  for (unsigned int i = 0 ; i < seq_size / 2 ; i++) {
+  //for (size_t i = 0 ; i < seq.size() ; i++) {
+  //  seq.setElement(i, NAR->translate(seq.getValue(i)));
+  //}
+  size_t seq_size = seq.size(); // store seq size for efficiency
+  int tmp_state = 0; // to store one state when swapping positions
+  size_t j = seq_size; // symetric position iterator from sequence end
+  for (size_t i = 0 ; i < seq_size / 2 ; i++) {
     j = seq_size - 1 - i;
     tmp_state = seq.getValue(i);
     seq.setElement(i, NAR->translate(seq.getValue(j)));
@@ -215,16 +216,29 @@ Sequence& SequenceTools::invertComplement(Sequence& seq) {
 
 /******************************************************************************/
 
-double SequenceTools::getPercentIdentity(const Sequence& seq1, const Sequence& seq2) throw (AlphabetMismatchException, SequenceNotAlignedException)
+double SequenceTools::getPercentIdentity(const Sequence& seq1, const Sequence& seq2, bool ignoreGaps) throw (AlphabetMismatchException, SequenceNotAlignedException)
 {
-	if(seq1.getAlphabet()->getAlphabetType() != seq2.getAlphabet()->getAlphabetType())
+	if (seq1.getAlphabet()->getAlphabetType() != seq2.getAlphabet()->getAlphabetType())
 		throw AlphabetMismatchException("SequenceTools::getPercentIdentity", seq1.getAlphabet(), seq2.getAlphabet());
-	if(seq1.size() != seq2.size())
+	if (seq1.size() != seq2.size())
 		throw SequenceNotAlignedException("SequenceTools::getPercentIdentity", &seq2);
-	int id = 0;
-	for(unsigned int i = 0; i < seq1.size(); i++)
-		if(seq1.getValue(i) == seq2.getValue(i)) id++;
-	return (double)id / (double)seq1.size() * 100.; 			
+  int gap = seq1.getAlphabet()->getGapCharacterCode();
+	unsigned int id = 0;
+	unsigned int tot = 0;
+	for (unsigned int i = 0; i < seq1.size(); i++) {
+    int x = seq1.getValue(i);
+    int y = seq2.getValue(i);
+    if (ignoreGaps) {
+      if (x != gap && y != gap) {
+        tot++;
+		    if (x == y) id++;
+      }
+    } else {
+      tot++;
+		  if (x == y) id++;
+    }
+  }
+	return static_cast<double>(id) / static_cast<double>(tot) * 100.; 			
 }
 
 /******************************************************************************/
@@ -255,7 +269,7 @@ unsigned int SequenceTools::getNumberOfCompleteSites(const Sequence & seq)
 
 /******************************************************************************/
 
-unsigned int SequenceTools::getNumberOfUnresolvedSites(const Sequence & seq)
+unsigned int SequenceTools::getNumberOfUnresolvedSites(const Sequence& seq)
 {
   unsigned int count = 0;
   const Alphabet * alpha = seq.getAlphabet();
@@ -268,21 +282,76 @@ unsigned int SequenceTools::getNumberOfUnresolvedSites(const Sequence & seq)
 
 /******************************************************************************/
 
-Sequence * SequenceTools::removeGaps(const Sequence & seq)
+Sequence* SequenceTools::getSequenceWithoutGaps(const Sequence& seq)
 {
+  const Alphabet* alpha = seq.getAlphabet();
 	vector<int> content;
-  const Alphabet * alpha = seq.getAlphabet();
-  for(unsigned int i = 0; i < seq.size(); i++)
+  for (unsigned int i = 0; i < seq.size(); i++)
   {
-		if(! alpha->isGap(seq[i])) content.push_back(seq[i]);
+		if (! alpha->isGap(seq[i])) content.push_back(seq[i]);
 	}
-  Sequence * newSeq = dynamic_cast<Sequence *>(seq.clone());
+  Sequence* newSeq = dynamic_cast<Sequence *>(seq.clone());
   newSeq->setContent(content);
 	return newSeq;
 }
 
 /******************************************************************************/
 
+void SequenceTools::removeGaps(Sequence& seq)
+{
+  const Alphabet* alpha = seq.getAlphabet();
+  for (unsigned int i = seq.size(); i > 0; --i)
+  {
+		if (alpha->isGap(seq[i - 1])) seq.deleteElement(i - 1);
+	}
+}
+
+/******************************************************************************/
+
+Sequence* SequenceTools::getSequenceWithoutStops(const Sequence& seq) throw (Exception)
+{
+  const CodonAlphabet* calpha = dynamic_cast<const CodonAlphabet*>(seq.getAlphabet());
+  if (!calpha)
+    throw Exception("SequenceTools::getSequenceWithoutStops. Input sequence should have a codon alphabet.");
+	vector<int> content;
+  for (unsigned int i = 0; i < seq.size(); i++)
+  {
+		if (!calpha->isStop(seq[i])) content.push_back(seq[i]);
+	}
+  Sequence* newSeq = dynamic_cast<Sequence*>(seq.clone());
+  newSeq->setContent(content);
+	return newSeq;
+}
+
+/******************************************************************************/
+
+void SequenceTools::removeStops(Sequence& seq) throw (Exception)
+{
+  const CodonAlphabet* calpha = dynamic_cast<const CodonAlphabet*>(seq.getAlphabet());
+  if (!calpha)
+    throw Exception("SequenceTools::removeStops. Input sequence should have a codon alphabet.");
+  for (unsigned int i = seq.size(); i > 0; --i)
+  {
+		if (calpha->isStop(seq[i - 1])) seq.deleteElement(i - 1);
+	}
+}
+
+/******************************************************************************/
+
+void SequenceTools::replaceStopsWithGaps(Sequence& seq) throw (Exception)
+{
+  const CodonAlphabet* calpha = dynamic_cast<const CodonAlphabet*>(seq.getAlphabet());
+  if (!calpha)
+    throw Exception("SequenceTools::replaceStopsWithGaps. Input sequence should have a codon alphabet.");
+  int gap = calpha->getGapCharacterCode();
+  for (unsigned int i = 0; i < seq.size(); ++i)
+  {
+		if (calpha->isStop(seq[i])) seq.setElement(i, gap);
+	}
+}
+
+/******************************************************************************/
+
 BowkerTest* SequenceTools::bowkerTest(const Sequence& seq1, const Sequence& seq2)
   throw (SequenceNotAlignedException)
 {
@@ -425,6 +494,86 @@ Sequence* SequenceTools::subtractHaplotype(const Sequence& s, const Sequence& h,
 
 /******************************************************************************/
 
+Sequence* SequenceTools::RNYslice(const Sequence& seq, int ph) throw (AlphabetException)
+{
+  // Alphabet type checking
+  if (seq.getAlphabet()->getAlphabetType() != "DNA alphabet")
+    {
+      throw AlphabetException ("SequenceTools::transcript : Sequence must be DNA", seq.getAlphabet());
+    }
+  
+  if ((ph<1) || (ph>3))
+    throw Exception("Bad phase for RNYSlice: " + TextTools::toString(ph) + ". Should be between 1 and 3.");
+  
+  int s = seq.size();
+  int n = (s - ph + 3) / 3;
+
+  vector<int> content(n);
+
+  int tir = seq.getAlphabet()->getGapCharacterCode();
+  int j;
+
+  for (int i = 0; i < n; i++)
+    {
+      j = i * 3 + ph - 1;
+
+      if (j == 0)
+        content[i] = _RNY.getRNY(tir,seq[0],seq[1],*seq.getAlphabet());
+      else
+        {
+          if (j == s - 1)
+            content[i] = _RNY.getRNY(seq[j - 1],seq[j],tir, *seq.getAlphabet());
+          else
+            content[i] = _RNY.getRNY(seq[j - 1],seq[j],seq[j + 1],*seq.getAlphabet());
+        }
+    }
+
+  // New sequence creating, and sense reversing
+  Sequence* sq = new BasicSequence(seq.getName(), content,
+                                   seq.getComments(), &_RNY);
+
+  // Send result
+  return sq;
+}
+
+Sequence* SequenceTools::RNYslice(const Sequence& seq) throw (AlphabetException)
+{
+  
+  // Alphabet type checking
+  if (seq.getAlphabet()->getAlphabetType() != "DNA alphabet")
+    {
+      throw AlphabetException ("SequenceTools::transcript : Sequence must be DNA", seq.getAlphabet());
+    }
+
+  unsigned int n = seq.size();
+
+  vector<int> content(n);
+
+  int tir = seq.getAlphabet()->getGapCharacterCode();
+
+  if (seq.size()>=2){
+    content[0] = _RNY.getRNY(tir,seq[0],seq[1], *seq.getAlphabet());
+
+    for (unsigned int i = 1; i < n - 1; i++)
+      {
+        content[i] = _RNY.getRNY(seq[i - 1],seq[i],seq[i + 1],
+                                 *seq.getAlphabet());
+      }
+    
+    content[n - 1] = _RNY.getRNY(seq[n - 2],seq[n - 1],tir, *seq.getAlphabet());
+  }
+  
+  // New sequence creating, and sense reversing
+  Sequence* s = new BasicSequence(seq.getName(), content,
+                                  seq.getComments(), &_RNY);
+
+  // Send result
+  return s;
+}
+
+/******************************************************************************/
+
+
 void SequenceTools::getCDS(Sequence& sequence, bool checkInit, bool checkStop, bool includeInit, bool includeStop)
 {
   const CodonAlphabet* alphabet = dynamic_cast<const CodonAlphabet*>(sequence.getAlphabet());
@@ -446,3 +595,24 @@ void SequenceTools::getCDS(Sequence& sequence, bool checkInit, bool checkStop, b
 
 /******************************************************************************/
 
+unsigned int SequenceTools::findFirstOf(const Sequence& seq, const Sequence& motif)
+{
+  if (motif.size() > seq.size())
+    return seq.size();
+  vector<int> tmp = motif.getContent();
+  deque<int> mint(tmp.begin(), tmp.end());
+  deque<int> window;
+  for (unsigned int i = 0; i < motif.size(); ++i)
+    window.push_back(seq[i]);
+  for (unsigned int i = 0; i <= seq.size() - motif.size(); ++i) {
+    if (window == mint)
+      return i;
+    //Move window:
+    window.push_back(seq[motif.size() + i]);
+    window.pop_front();
+  }
+  return seq.size();
+}
+
+/******************************************************************************/
+
